// Description: creates a transform tracked with sensor 0 and a ygHead child node
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> this node can also be created with the alias: CAVEHead
//<li> this node should be a child of a <a href="ygUser.html">ygUser</a> node
//<li> sets the head of the user above it for use in collision detection and user detection
//<li> this node is required for walking navigation and collision detection
//</ul>
//
// Category: User
// Author: Dave Pape
//           11/01/01
// Revision: 09/01/04 Alex Hill - automatically create a child ygHead node (deprecated)
//
#include <pfcave.h>
#include "ygUser.h"
#include "ygNodeDB.h"
#include "ygHead.h"
#include "ygCAVEHead.h"

extern "C" ygNode* construct_ygCAVEHead(const char* name,bool master) { return new ygCAVEHead(name,master); }

struct _ygCAVEHeadPrivateData
    {
    ygHead* head;
    };


ygCAVEHead::ygCAVEHead(const char* name,bool master) : ygCAVETracker(name,master)
    {
    setClassName("ygCAVEHead");
    p_ = new struct _ygCAVEHeadPrivateData;
    ygString headName;
    ygNodeDB::createValidName("ygHead",headName);
    p_->head = (ygHead*)ygNodeDB::create("ygHead",headName);
    addChild(p_->head);
    }


void ygCAVEHead::reset(void)
    {
    p_->head->reset();
    ygCAVETracker::reset();
    }
