// Description: establishes a view point for the CAVE library rendering
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> creates a tracked transform from CAVE sensor 0
//<li> added automatically to scene at startup unless the --noview option is used
//</ul>
//
// Category: Foundation
// Author: Dave Pape
//           11/01/01
// Revision: 05/01/03 Alex Hill - added setUser() method to use another user perspective
//
#include <pfcave.h>
#include "ygWorld.h"
#include "ygCAVEViewer.h"
#include "ygNavigator.h"
#include "ygUser.h"
#include "ygNodeDB.h"

extern "C" ygNode* construct_ygCAVEViewer(const char* name,bool master) { return new ygCAVEViewer(name,master); }


ygCAVEViewer::ygCAVEViewer(const char* name,bool master) : ygViewer(name,master)
    {
    setClassName("ygCAVEViewer");
    user_ = NULL;
    CAVESetOption(CAVE_PROJ_INCLUDENAVIGATION,1);
    }


void ygCAVEViewer::reset(void)
    {
    ygViewer::reset();
    }


void ygCAVEViewer::app(void)
    {
    if (navigator())
        {
        CAVENavLock();
        CAVENavLoadIdentity();
        CAVENavTranslate(navigator()->position()[0],
                         navigator()->position()[1],
                         navigator()->position()[2]);
        CAVENavRot(navigator()->orientation()[PF_Z],'z');
        CAVENavRot(navigator()->orientation()[PF_X],'x');
        CAVENavRot(navigator()->orientation()[PF_Y],'y');
        CAVENavUnlock();
        }
    //if user and associated head are found then
    if (user_ && user_->head())
        {
        pfMatrix userMat;
        //get user navigation matrix
        user_->getMatrix(userMat);
        pfCoord coord;
        userMat.getOrthoCoord(&coord);
        //update navigation data
        CAVENavLock();
        CAVENavLoadIdentity();
        CAVENavTranslate(coord.xyz.vec[0],coord.xyz.vec[1],coord.xyz.vec[2]);
        CAVENavRot(coord.hpr.vec[0],'z');
        CAVENavRot(coord.hpr.vec[1],'x');
        CAVENavRot(coord.hpr.vec[2],'y');
        CAVENavUnlock();
        //get user head matrix
        pfMatrix headMat;
        user_->head()->getTransform(headMat);
        pfMatrix userInv;
        userInv.invertOrtho(userMat);
        headMat.postMult(userInv);
        headMat.getOrthoCoord(&coord);
        //update head sensor
        CAVESensor1->x = coord.xyz.vec[0];
        CAVESensor1->y = coord.xyz.vec[1];
        CAVESensor1->z = coord.xyz.vec[2];
        CAVESensor1->azim = coord.hpr.vec[0];
        CAVESensor1->elev = coord.hpr.vec[1];
        CAVESensor1->roll = coord.hpr.vec[2];
        //get navigated head position
        }
    pfVec3 pos, ori;
    //get CAVE head position
    CAVEGetPosition(CAVE_HEAD_NAV, pos.vec);
    //get CAVE head orientation
    CAVEGetOrientation(CAVE_HEAD_NAV, ori.vec);
    //set the position and orientation of the transform
    setPosition(pos);
    setOrientation(ori);
    ygViewer::app();
    }



void ygCAVEViewer::setUser(ygUser* user)
    {
    user_ = user;
    if (user_)
        {
        tracking_ = CAVEConfig->UseTracker;
        ygMessage msg;
        msg.parseString("updateInterval(10)",NULL,NULL);
        CAVEConfig->UseTracker = 0;
        }
    else
        {
        ygMessage msg;
        msg.parseString("updateInterval(-1)",NULL,NULL);
        CAVEConfig->UseTracker = tracking_;
        }    
    }
