#ifndef _ygMessage_h_
#define _ygMessage_h_

#include <ctype.h>
#include <stdlib.h>
#include <iostream>
#include <vector>
#include <Performer/pr/pfLinMath.h>
#include "ygString.h"
class ygNode;
class ygEvent;


class ygMessage
    {
    public:
     inline const ygString& message(void) const
        { return message_; }
     inline int numArgs(void) const
        { return args_.size(); }
     inline int numArgTypes(void) const
        { return argTypes_.size(); }
     inline float delay(void) const
        { return delay_; }
     inline ygNode * recipient(void) const
        { return recipient_; }
     inline const ygString& recipientName(void) const
        { return recipientName_; }

     friend inline bool operator==(const ygMessage& a,const ygString& b)
        { return a.message_ == b; }
     friend inline bool operator==(const ygString& a,const ygMessage& b)
        { return a == b.message_; }
     inline int typeArg(int i) const
        { return argTypes_[i]; }
     inline const ygString& stringArg(int i) const
        { return args_[i]; }
     inline float floatArg(int i) const
        {
        if (args_.size() > i)
            {
            if (!isalpha(*(args_[i].c_str())))
                return atof(args_[i].c_str());
            else if ((args_[i] == "true") || (args_[i] == "on"))
                return 1.0;
            else
                return 0.0;
            }
        else
            return 0.0;
        }
     inline int intArg(int i) const
        {
        if (args_.size() > i)
            return atoi(args_[i].c_str());
        else
            return 0;
        }
     inline bool boolArg(int i) const
        {
        if (args_.size() <= i)
            return false;
        else if ((args_[i] == "true") || (args_[i] == "on") ||
            (args_[i] == "1") || (args_[i] == "y") || (args_[i] == "yes"))
            return true;
        else
            return false;
        }
     inline bool getVec2Args(pfVec2& v,int i=0) const
        {
        if (args_.size() >= 2+i)
            {
            v[0] = atof(args_[0+i].c_str());
            v[1] = atof(args_[1+i].c_str());
            return true;
            }
        else
            return false;
        }
     inline bool getVec3Args(pfVec3& v,int i=0) const
        {
        if (args_.size() >= 3+i)
            {
            v[0] = atof(args_[0+i].c_str());
            v[1] = atof(args_[1+i].c_str());
            v[2] = atof(args_[2+i].c_str());
            return true;
            }
        else
            return false;
        }
     inline bool getVec4Args(pfVec4& v,int i=0) const
        {
        if (args_.size() >= 4+i)
            {
            v[0] = atof(args_[0+i].c_str());
            v[1] = atof(args_[1+i].c_str());
            v[2] = atof(args_[2+i].c_str());
            v[3] = atof(args_[3+i].c_str());
            return true;
            }
        else
            return false;
        }
     inline pfVec2 vec2Args(int i=0) const
        {
        if (args_.size() >= 2+i)
            {
            float v[2];
            v[0] = atof(args_[0+i].c_str());
            v[1] = atof(args_[1+i].c_str());
            return pfVec2(v[0],v[1]);
            }
        else
            return pfVec2(0,0);
        }
     inline pfVec3 vec3Args(int i=0) const
        {
        if (args_.size() >= 3+i)
            {
            float v[3];
            v[0] = atof(args_[0+i].c_str());
            v[1] = atof(args_[1+i].c_str());
            v[2] = atof(args_[2+i].c_str());
            return pfVec3(v[0],v[1],v[2]);
            }
        else
            return pfVec3(0,0,0);
        }
     inline pfVec4 vec4Args(int i=0) const
        {
        if (args_.size() >= 4+i)
            {
            float v[4];
            v[0] = atof(args_[0+i].c_str());
            v[1] = atof(args_[1+i].c_str());
            v[2] = atof(args_[2+i].c_str());
            v[3] = atof(args_[3+i].c_str());
            return pfVec4(v[0],v[1],v[2],v[3]);
            }
        else
            return pfVec4(0,0,0,0);
        }

     friend std::ostream& operator<<(std::ostream& s,const ygMessage& m);
     friend class ygNetMessages;

     ygMessage(void);
     ygMessage(const ygString&);
     ~ygMessage(void);
     ygMessage& operator=(const ygMessage&);
     void parseString(const ygString&,ygNode *n=NULL,ygEvent* e=NULL);
     void writeString(ygString&,bool all=true) const;
     void replaceNodeName(const ygString&,const ygString&);
     void error(const ygString& prefix,const ygString& suffix="") const;

     inline void setRecipient(ygNode *n)
        { recipient_ = n; }

    private:
     ygString message_;
     std::vector<ygString> args_;
     std::vector<int> argTypes_;
     ygNode * recipient_;
     ygString recipientName_;
     float delay_;

     void replaceArgs(ygString& str,ygNode*,ygEvent*);
     void splitMessageName(const ygString&,ygNode *);
    };

#endif
